#include "cgreeble.h"
#include <stdio.h>

//----------------------------------------------------------------------
CGreeble::CGreeble()
{
   mObject = NULL;
   mRotationInc = 90.0f;
   mScaleMin = 1.0f;
   mScaleMax = 1.0f;
   mYheight = 0.0f;
}
//----------------------------------------------------------------------
CGreeble::~CGreeble()
{
   ClearObject();
}
//----------------------------------------------------------------------
void CGreeble::ClearObject()
{
   if( mObject != NULL )
   {
      object_delete( mObject );
      mObject = NULL;
      ClearGreObject();
   }
   mYheight = 0.0f;
}
//----------------------------------------------------------------------
void CGreeble::CreateCubic()
{

   ClearObject();

   mIsCustom = false;

   mObject = ac_new_object();

   // Create vertices
   Vertex *v[8];
   v[0] = new_vertex_set( -0.5f, 0.0f,  0.5f );
   v[1] = new_vertex_set(  0.5f, 0.0f,  0.5f );
   v[2] = new_vertex_set(  0.5f, 0.0f, -0.5f );
   v[3] = new_vertex_set( -0.5f, 0.0f, -0.5f );
   v[4] = new_vertex_set( -0.5f, 1.0f,  0.5f );
   v[5] = new_vertex_set(  0.5f, 1.0f,  0.5f );
   v[6] = new_vertex_set(  0.5f, 1.0f, -0.5f );
   v[7] = new_vertex_set( -0.5f, 1.0f, -0.5f );

   // Add vertices to object
   for( int ix = 0; ix < 8; ++ix )
   {
      object_add_vertex_head( mObject, v[ix] );
   }

   // Add vertices to surfaces, add surfaces to object
   Surface *s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[0], 0.0f, 1.0f );
   surface_add_vertex( s, v[1], 1.0f, 1.0f );
   surface_add_vertex( s, v[5], 1.0f, 0.0f );
   surface_add_vertex( s, v[4], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[1], 0.0f, 1.0f );
   surface_add_vertex( s, v[2], 1.0f, 1.0f );
   surface_add_vertex( s, v[6], 1.0f, 0.0f );
   surface_add_vertex( s, v[5], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[2], 0.0f, 1.0f );
   surface_add_vertex( s, v[3], 1.0f, 1.0f );
   surface_add_vertex( s, v[7], 1.0f, 0.0f );
   surface_add_vertex( s, v[6], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[3], 0.0f, 1.0f );
   surface_add_vertex( s, v[0], 1.0f, 1.0f );
   surface_add_vertex( s, v[4], 1.0f, 0.0f );
   surface_add_vertex( s, v[7], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[4], 0.0f, 1.0f );
   surface_add_vertex( s, v[5], 1.0f, 1.0f );
   surface_add_vertex( s, v[6], 1.0f, 0.0f );
   surface_add_vertex( s, v[7], 0.0f, 0.0f );

   mYheight = 1.0f;

   // Push the surfaces/verts into local arrays
   PopulateVSArray();

}
//----------------------------------------------------------------------
void CGreeble::CreateL()
{

   ClearObject();

   mIsCustom = false;

   mObject = ac_new_object();

   // Create vertices
   int num_sides = 6;
   int num_verts = 12;
   Vertex *v[12];
   v[0]  = new_vertex_set( -0.5f, 0.0f,  0.5f );
   v[1]  = new_vertex_set(  0.5f, 0.0f,  0.5f );
   v[2]  = new_vertex_set(  0.5f, 0.0f,  0.0f );
   v[3]  = new_vertex_set(  0.0f, 0.0f,  0.0f );
   v[4]  = new_vertex_set(  0.0f, 0.0f, -0.5f );
   v[5]  = new_vertex_set( -0.5f, 0.0f, -0.5f );
   v[6]  = new_vertex_set( -0.5f, 1.0f,  0.5f );
   v[7]  = new_vertex_set(  0.5f, 1.0f,  0.5f );
   v[8]  = new_vertex_set(  0.5f, 1.0f,  0.0f );
   v[9]  = new_vertex_set(  0.0f, 1.0f,  0.0f );
   v[10] = new_vertex_set(  0.0f, 1.0f, -0.5f );
   v[11] = new_vertex_set( -0.5f, 1.0f, -0.5f );

   // Add vertices to object
   for( int ix = 0; ix < num_verts; ++ix )
   {
      object_add_vertex_head( mObject, v[ix] );
   }

   // Add vertices to surfaces, add surfaces to object
   Surface *s;
   // Sides
   for( int ix = 0; ix < num_sides; ++ix )
   {
      s = new_surface();
      object_add_surface_head( mObject, s );
      if( ix + 1 < num_sides )
      {
         surface_add_vertex( s, v[ix], 0.0f, 1.0f );
         surface_add_vertex( s, v[ix+1], 1.0f, 1.0f );
         surface_add_vertex( s, v[ix+num_sides+1], 1.0f, 0.0f );
         surface_add_vertex( s, v[ix+num_sides], 0.0f, 0.0f );
      }
      else
      {
         surface_add_vertex( s, v[ix], 0.0f, 1.0f );
         surface_add_vertex( s, v[0], 1.0f, 1.0f );
         surface_add_vertex( s, v[num_sides], 1.0f, 0.0f );
         surface_add_vertex( s, v[ix+num_sides], 0.0f, 0.0f );
      }
   }
   // Top
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[6], 0.0f, 1.0f );
   surface_add_vertex( s, v[7], 1.0f, 1.0f );
   surface_add_vertex( s, v[8], 1.0f, 0.5f );
   surface_add_vertex( s, v[9], 0.5f, 0.5f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[9], 0.5f, 0.5f );
   surface_add_vertex( s, v[10], 0.5f, 0.0f );
   surface_add_vertex( s, v[11], 0.0f, 0.0f );
   surface_add_vertex( s, v[6], 0.0f, 1.0f );

   mYheight = 1.0f;

   // Push the surfaces/verts into local arrays
   PopulateVSArray();

}
//----------------------------------------------------------------------
void CGreeble::CreateT()
{

   ClearObject();

   mIsCustom = false;

   mObject = ac_new_object();

   // Create vertices
   int num_sides = 8;
   int num_verts = 16;
   Vertex *v[16];
   float one_third = 1.0f / 3.0f;
   v[0]  = new_vertex_set( -0.5f + one_third, 0.0f,  0.5f );
   v[1]  = new_vertex_set(  0.5f - one_third, 0.0f,  0.5f );
   v[2]  = new_vertex_set(  0.5f - one_third, 0.0f,  0.0f );
   v[3]  = new_vertex_set(  0.5f,             0.0f,  0.0f );
   v[4]  = new_vertex_set(  0.5f,             0.0f, -0.5f );
   v[5]  = new_vertex_set( -0.5f,             0.0f, -0.5f );
   v[6]  = new_vertex_set( -0.5f,             0.0f,  0.0f );
   v[7]  = new_vertex_set( -0.5f + one_third, 0.0f,  0.0f );
   v[8]  = new_vertex_set( -0.5f + one_third, 1.0f,  0.5f );
   v[9]  = new_vertex_set(  0.5f - one_third, 1.0f,  0.5f );
   v[10] = new_vertex_set(  0.5f - one_third, 1.0f,  0.0f );
   v[11] = new_vertex_set(  0.5f,             1.0f,  0.0f );
   v[12] = new_vertex_set(  0.5f,             1.0f, -0.5f );
   v[13] = new_vertex_set( -0.5f,             1.0f, -0.5f );
   v[14] = new_vertex_set( -0.5f,             1.0f,  0.0f );
   v[15] = new_vertex_set( -0.5f + one_third, 1.0f,  0.0f );

   // Add vertices to object
   for( int ix = 0; ix < num_verts; ++ix )
   {
      object_add_vertex_head( mObject, v[ix] );
   }

   // Add vertices to surfaces, add surfaces to object
   Surface *s;
   // Sides
   for( int ix = 0; ix < num_sides; ++ix )
   {
      s = new_surface();
      object_add_surface_head( mObject, s );
      if( ix + 1 < num_sides )
      {
         surface_add_vertex( s, v[ix], 0.0f, 1.0f );
         surface_add_vertex( s, v[ix+1], 1.0f, 1.0f );
         surface_add_vertex( s, v[ix+num_sides+1], 1.0f, 0.0f );
         surface_add_vertex( s, v[ix+num_sides], 0.0f, 0.0f );
      }
      else
      {
         surface_add_vertex( s, v[ix], 0.0f, 1.0f );
         surface_add_vertex( s, v[0], 1.0f, 1.0f );
         surface_add_vertex( s, v[num_sides], 1.0f, 0.0f );
         surface_add_vertex( s, v[ix+num_sides], 0.0f, 0.0f );
      }
   }
   // Top
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[8], one_third, 1.0f );
   surface_add_vertex( s, v[9], 1.0f - one_third, 1.0f );
   surface_add_vertex( s, v[10], 1.0f - one_third, 0.5f );
   surface_add_vertex( s, v[15], one_third, 0.5f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[10], 1.0f - one_third, 0.5f );
   surface_add_vertex( s, v[11], 1.0f, 0.5f );
   surface_add_vertex( s, v[12], 1.0f, 0.0f );
   surface_add_vertex( s, v[13], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[13], 0.0f, 0.0f );
   surface_add_vertex( s, v[14], 0.0f, 0.5f );
   surface_add_vertex( s, v[15], one_third, 0.5f );
   surface_add_vertex( s, v[10], 1.0f - one_third, 0.5f );

   mYheight = 1.0f;

   // Push the surfaces/verts into local arrays
   PopulateVSArray();

}
//----------------------------------------------------------------------
void CGreeble::CreateIII()
{

   ClearObject();

   mIsCustom = false;

   mObject = ac_new_object();

   // Create vertices
   Vertex *v[24];
   v[0]  = new_vertex_set( -0.4f, 0.0f,  0.4f );
   v[1]  = new_vertex_set( -0.2f, 0.0f,  0.4f );
   v[2]  = new_vertex_set( -0.2f, 0.0f, -0.4f );
   v[3]  = new_vertex_set( -0.4f, 0.0f, -0.4f );
   v[4]  = new_vertex_set( -0.1f, 0.0f,  0.4f );
   v[5]  = new_vertex_set(  0.1f, 0.0f,  0.4f );
   v[6]  = new_vertex_set(  0.1f, 0.0f, -0.4f );
   v[7]  = new_vertex_set( -0.1f, 0.0f, -0.4f );
   v[8]  = new_vertex_set(  0.2f, 0.0f,  0.4f );
   v[9]  = new_vertex_set(  0.4f, 0.0f,  0.4f );
   v[10] = new_vertex_set(  0.4f, 0.0f, -0.4f );
   v[11] = new_vertex_set(  0.2f, 0.0f, -0.4f );
   v[12] = new_vertex_set( -0.4f, 1.0f,  0.4f );
   v[13] = new_vertex_set( -0.2f, 1.0f,  0.4f );
   v[14] = new_vertex_set( -0.2f, 1.0f, -0.4f );
   v[15] = new_vertex_set( -0.4f, 1.0f, -0.4f );
   v[16] = new_vertex_set( -0.1f, 1.0f,  0.4f );
   v[17] = new_vertex_set(  0.1f, 1.0f,  0.4f );
   v[18] = new_vertex_set(  0.1f, 1.0f, -0.4f );
   v[19] = new_vertex_set( -0.1f, 1.0f, -0.4f );
   v[20] = new_vertex_set(  0.2f, 1.0f,  0.4f );
   v[21] = new_vertex_set(  0.4f, 1.0f,  0.4f );
   v[22] = new_vertex_set(  0.4f, 1.0f, -0.4f );
   v[23] = new_vertex_set(  0.2f, 1.0f, -0.4f );

   // Add vertices to object
   for( int ix = 0; ix < 24; ++ix )
   {
      object_add_vertex_head( mObject, v[ix] );
   }

#define TMP_ADD_SURF(A,B,C,D) { s = new_surface(); object_add_surface_head( mObject, s ); surface_add_vertex( s, v[A], 0.0f, 1.0f ); surface_add_vertex( s, v[B], 1.0f, 1.0f ); surface_add_vertex( s, v[C], 1.0f, 0.0f ); surface_add_vertex( s, v[D], 0.0f, 0.0f ); }
   // Add vertices to surfaces, add surfaces to object
   Surface *s;
   TMP_ADD_SURF(  0,  1, 13, 12 );
   TMP_ADD_SURF(  1,  2, 14, 13 );
   TMP_ADD_SURF(  2,  3, 15, 14 );
   TMP_ADD_SURF(  3,  0, 12, 15 );
   TMP_ADD_SURF(  4,  5, 17, 16 );
   TMP_ADD_SURF(  5,  6, 18, 17 );
   TMP_ADD_SURF(  6,  7, 19, 18 );
   TMP_ADD_SURF(  7,  4, 16, 19 );
   TMP_ADD_SURF(  8,  9, 21, 20 );
   TMP_ADD_SURF(  9, 10, 22, 21 );
   TMP_ADD_SURF( 10, 11, 23, 22 );
   TMP_ADD_SURF( 11,  8, 20, 23 );
#undef TMP_ADD_SURF

   // Tops
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[12], 0.0f, 1.0f );
   surface_add_vertex( s, v[13], 1.0f, 1.0f );
   surface_add_vertex( s, v[14], 1.0f, 0.0f );
   surface_add_vertex( s, v[15], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[16], 0.0f, 1.0f );
   surface_add_vertex( s, v[17], 1.0f, 1.0f );
   surface_add_vertex( s, v[18], 1.0f, 0.0f );
   surface_add_vertex( s, v[19], 0.0f, 0.0f );
   s = new_surface();
   object_add_surface_head( mObject, s );
   surface_add_vertex( s, v[20], 0.0f, 1.0f );
   surface_add_vertex( s, v[21], 1.0f, 1.0f );
   surface_add_vertex( s, v[22], 1.0f, 0.0f );
   surface_add_vertex( s, v[23], 0.0f, 0.0f );

   mYheight = 1.0f;

   // Push the surfaces/verts into local arrays
   PopulateVSArray();

}
//----------------------------------------------------------------------
void CGreeble::LoadFromObject( ACObject *obj )
{

   ClearObject();

   mIsCustom = true;

   // Clone the object
   mObject = object_clone( obj );

   // Translate the greeble to be centered at 0.0/0.0 X/Z, and to rest on the 0.0 Y axis
   Vertex bbox_min, bbox_max;
   bool first = true;
   for( List *pv = ac_object_get_vertexlist( mObject ); pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      if( first )
      {
         bbox_min.x = v->x;
         bbox_min.y = v->y;
         bbox_min.z = v->z;
         bbox_max.x = v->x;
         bbox_max.y = v->y;
         bbox_max.z = v->z;
         first = false;
      }
      else
      {
         bbox_min.x = min( bbox_min.x, v->x );
         bbox_min.y = min( bbox_min.y, v->y );
         bbox_min.z = min( bbox_min.z, v->z );
         bbox_max.x = max( bbox_max.x, v->x );
         bbox_max.y = max( bbox_max.y, v->y );
         bbox_max.z = max( bbox_max.z, v->z );
      }
   }

   // Calculate center and offset
   Vertex bbox_center, bbox_offset;
   bbox_center.x = bbox_min.x + (( bbox_max.x - bbox_min.x ) / 2.0f );
   bbox_center.y = bbox_min.y + (( bbox_max.y - bbox_min.y ) / 2.0f );
   bbox_center.z = bbox_min.z + (( bbox_max.z - bbox_min.z ) / 2.0f );
   bbox_offset.x = -( bbox_center.x );
   bbox_offset.y = -( bbox_min.y );
   bbox_offset.z = -( bbox_center.z );

   mYheight = bbox_max.y + bbox_offset.y;

   // Center the object
   for( List *pv = ac_object_get_vertexlist( mObject ); pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      v->x += bbox_offset.x;
      v->y += bbox_offset.y;
      v->z += bbox_offset.z;
   }

   // Set the texture index
   if( ac_object_has_texture( obj ))
   {
      mTexIndex = ac_object_get_texture_index( obj );
   }
   else
   {
       mTexIndex = -1;
   }

   // Push the surfaces/verts into local arrays
   PopulateVSArray();

}
//----------------------------------------------------------------------
void CGreeble::PopulateVSArray()
{
   mNumVerts = list_count( ac_object_get_vertexlist( mObject ));
   mNumSurfs = list_count( ac_object_get_surfacelist( mObject ));
   mVerts = new CGreVertex[mNumVerts];
   mSurfs = new CGreSurface[mNumSurfs];
   int ctr = 0;
   for( List *pv = ac_object_get_vertexlist( mObject ); pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      mVerts[ctr].v.x = v->x;
      mVerts[ctr].v.y = v->y;
      mVerts[ctr].v.z = v->z;
      mVerts[ctr].ac_v = v;
      ++ctr;
   }
   ctr = 0;
   for( List *ps = ac_object_get_surfacelist( mObject ); ps != NULL; ps = ps->next )
   {
      Surface *s = ( Surface * )( ps->data );
      mSurfs[ctr].numverts = s->numvert;
      mSurfs[ctr].sverts = new CGreSVertex[s->numvert];
      int ctr2 = 0;
      for( List *psv = s->vertlist; psv != NULL; psv = psv->next )
      {
         SVertex *sv = ( SVertex * )( psv->data );
         mSurfs[ctr].sverts[ctr2].tx = sv->tx;
         mSurfs[ctr].sverts[ctr2].ty = sv->ty;
         for( unsigned int ix = 0; ix < mNumVerts; ++ix )
         {
            if( mVerts[ix].ac_v == sv->v )
            {
               mSurfs[ctr].sverts[ctr2].v = ix;
               break;
            }
         }
         ++ctr2;
      }
      ++ctr;
   }
   FindBounds();
}
//----------------------------------------------------------------------
