#ifndef u_cgreebler_H
#define u_cgreebler_H

#include "cgreeble.h"
#include "ctargetsurface.h"

//----------------------------------------------------------------------
class CGreebler
{

public:

   // Greeble collection
   CGreeble *mGreebles;
   unsigned int mNumGreebles;

   // Random seed
   unsigned int mRandSeed;

   // Size of a small poly.
   // For tris, one side is <= this size.
   // For quads, two opposing sides are <= this size.
   float mSmallPolySize;

   // Keep bevels/greebles in separate objects
   bool mObjKeepSeparate;

   //////////////////////////////////////////////////////////////////////
   // Bevel params

   // Bevel?
   bool mDoBevel;
   // Randomly bevel?
   bool mBevRandom;
   float mBevRandomPct;
   // Bevel distance
   float mBevDistancePctMin;
   float mBevDistancePctMax;
   // Bevel indent
   float mBevIndentPctMin;
   float mBevIndentPctMax;
   // Percent chance that a bevel will be split into 1-4 quadrants
   float mBevGridPct;

   //////////////////////////////////////////////////////////////////////
   // Greeble params

   // Greeble?
   bool mDoGreeble;
   // Randomly greeble?
   bool mGreRandom;
   float mGreRandomPct;
   // Num greebles per surface
   unsigned short mGreNumPerSurfaceMin;
   unsigned short mGreNumPerSurfaceMax;
   // Greeble height
   float mGreHeightPctMin;
   float mGreHeightPctMax;
   // Size
   float mGreSizePctMin;
   float mGreSizePctMax;
   // Rotation increments
   float mGreRotIncrDegrees;
   // Do not conform custom greeble to shapes to the underlying surface
   bool mGreCustNoConform;

   //////////////////////////////////////////////////////////////////////
   // Custom params

   bool mUseCustom;
   bool mUseStandard;

   //////////////////////////////////////////////////////////////////////

   // Con/Destructor
   CGreebler();
   ~CGreebler();

   // Clear greeble collection
   void ClearGreebleCollection();

   // Generate greebles
   List *Generate( bool add_undoable );

   // Bevel a surface and return the end surface
   Surface *BevelSurface( List **newobjs, List **newsurfs, List **newverts, Surface *surf, int numverts, float xarr[], float yarr[], float zarr[] );

   // Greeble a surface
   void GreebleSurface( List **newobjs, List **newsurfs, List **newverts, CTargetSurface *surf, CGreeble *in_greeble );

};
//----------------------------------------------------------------------

#endif
