#ifndef u_ctargetsurface_H
#define u_ctargetsurface_H

#include "c3d.h"
#undef MIN
#undef MAX
#include "geometryc.h"
#include <stdio.h>

#define NUM_POINTS 4
//----------------------------------------------------------------------
class CTargetSurface
{

private:

   void XFormRotateAxis( Vertex *center, Vertex *axis, double C )
   {
      double S;
      double a = axis->x,
             b = axis->y,
             c = axis->z;
      double x0 = center->x,
             y0 = center->y,
             z0 = center->z;
      if( C < -1.0 )
      {
         C = -1.0;
      }
      else if( C > 1.0 )
      {
         C = 1.0;
      }
      S = sqrt( 1.0 - C * C );
      for( int ix = 0; ix < NUM_POINTS; ++ix )
      {
         Vertex *v = &( xform_points[ix] );
         double x = v->x - x0,
                y = v->y - y0,
                z = v->z - z0;
         v->x = ( float )((( 1.0 - C ) * a * a + C     ) * x + (( 1.0 - C ) * b * a - c * S ) * y + (( 1.0 - C ) * c * a + b * S ) * z + x0 );
         v->y = ( float )((( 1.0 - C ) * a * b + c * S ) * x + (( 1.0 - C ) * b * b + C     ) * y + (( 1.0 - C ) * c * b - a * S ) * z + y0 );
         v->z = ( float )((( 1.0 - C ) * a * c - b * S ) * x + (( 1.0 - C ) * b * c + a * S ) * y + (( 1.0 - C ) * c * c + C     ) * z + z0 );
      }
   }

public:

   Surface *acsurf;
   Vertex xform_points[NUM_POINTS];
   float surf_area;
   Vertex surf_center;
   float rot_xz;
   Vertex xform_min, xform_max;

   // Constructor
   CTargetSurface()
   {
      acsurf = NULL;
   }

   // Get/prepare the surface
   void PrepareSurface( Surface *s )
   {
      acsurf = s;
      // Push surface vertices into points array
      int pts = 0;
      for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
      {
         SVertex *sv = ( SVertex * )( pv->data );
         xform_points[pts].x = sv->v->x;
         xform_points[pts].y = sv->v->y;
         xform_points[pts].z = sv->v->z;
         ++pts;
      }
      while( pts < NUM_POINTS )
      {
         // Grab first vertex into unspecified slots.
         // This allows 3-sided surfaces to be treated as 4-sided surfaces.
         SVertex *sv = ( SVertex * )( s->vertlist->data );
         xform_points[pts].x = sv->v->x;
         xform_points[pts].y = sv->v->y;
         xform_points[pts].z = sv->v->z;
         ++pts;
      }
      // Calculate the area of the surface
      float x[NUM_POINTS], y[NUM_POINTS], z[NUM_POINTS];
      for( int ix = 0; ix < NUM_POINTS; ++ix )
      {
         x[ix] = xform_points[ix].x;
         y[ix] = xform_points[ix].y;
         z[ix] = xform_points[ix].z;
      }
      surf_area = polygon_area_2_3d( NUM_POINTS, x, y, z );
      // Get poly center
      float xarr[NUM_POINTS], yarr[NUM_POINTS], zarr[NUM_POINTS];
      for( int ix = 0; ix < NUM_POINTS; ++ix )
      {
         xarr[ix] = xform_points[ix].x;
         yarr[ix] = xform_points[ix].y;
         zarr[ix] = xform_points[ix].z;
      }
      polygon_centroid_3d( NUM_POINTS, xarr, yarr, zarr, &( surf_center.x ), &( surf_center.y ), &( surf_center.z ));
      // Rotate surface points to point along positive Y axis
      Vertex axis;
      Vertex up;
      up.x = 0;
      up.y = 1;
      up.z = 0;
      Vertex norm;
      norm.x = s->normal.x;
      norm.y = s->normal.y;
      norm.z = s->normal.z;
      C3d::VectorAxis( &norm, &up, &axis );
      float cosa = C3d::VectorCosAngle( &up, &norm );
      XFormRotateAxis( &surf_center, &axis, cosa );
      // Move the surface to 0,0,0
      for( int ix = 0; ix < NUM_POINTS; ++ix )
      {
         xform_points[ix].x -= surf_center.x;
         xform_points[ix].y -= surf_center.y;
         xform_points[ix].z -= surf_center.z;
      }
      // Rotate on Y axis so first polygon side is horizontal
      // Based on this operation, xform_points[0] will be the "bottom-right" vertex on the XZ plane.
      rot_xz = angle_rad_2d( xform_points[0].x, xform_points[0].z,
                             xform_points[1].x, xform_points[1].z,
                             xform_points[1].x - 1.0f, xform_points[1].z );
      for( unsigned int ix = 0; ix < NUM_POINTS; ++ix )
      {
         float newx = xform_points[ix].x * cos( rot_xz ) - xform_points[ix].z * sin( rot_xz );
         float newz = xform_points[ix].x * sin( rot_xz ) + xform_points[ix].z * cos( rot_xz );
         xform_points[ix].x = newx;
         xform_points[ix].z = newz;
      }
      // Get the XZ size
      xform_min.y = 0.0f;
      xform_max.y = 0.0f;
      for( int ix = 0; ix < NUM_POINTS; ++ix )
      {
         if( ix == 0 )
         {
            xform_min.x = xform_points[ix].x;
            xform_min.z = xform_points[ix].z;
            xform_max.x = xform_points[ix].x;
            xform_max.z = xform_points[ix].z;
         }
         else
         {
            xform_min.x = min( xform_min.x, xform_points[ix].x );
            xform_min.z = min( xform_min.z, xform_points[ix].z );
            xform_max.x = max( xform_max.x, xform_points[ix].x );
            xform_max.z = max( xform_max.z, xform_points[ix].z );
         }
      }
   }

};
//----------------------------------------------------------------------
#undef NUM_POINTS

#endif
