#include "cgreebler.h"
#include <stdio.h>
#include <string.h>

// TODO: Revert this to g_DDHGreebleCreatedObject (not a list) if Andy comes back and says cleanup is performed correctly with object_delete().
// Otherwise, we'll need to delete the individual objects under the group.
static List *g_DDHGreebleCreatedObjectList = NULL;
static bool g_DDHGreebleStoreSettings = false;

//---------------------------------------------------------------------------------------
// Disable realtime updates
static void ddhgreeble_disable_realtime()
{
   tcl_command( "grab release .ddhgreeble" );
   // Remove the old object
   ACObject *obj = NULL;
   if( g_DDHGreebleCreatedObjectList != NULL )
   {
      obj = ( ACObject * )( g_DDHGreebleCreatedObjectList->data );
   }
   else
   {
      return;
   }
   if( object_exists( obj ))
   {
      // Only need to delete the first object in the list, as it's the group container
      object_delete( obj );
      redraw_all();
      display_status();
   }
   list_free( &g_DDHGreebleCreatedObjectList );
   g_DDHGreebleCreatedObjectList = NULL;
}
//---------------------------------------------------------------------------------------
// Enable realtime updates
static void ddhgreeble_enable_realtime()
{
   tcl_command( "grab .ddhgreeble" );
}
//---------------------------------------------------------------------------------------
static void ddhgreeble_set_save_options( int argc, const char* argv[] )
{
   if( argc == 2 )
   {
      if( argv[1][0] == '0' && argv[1][1] == '\0' )
      {
         g_DDHGreebleStoreSettings = false;
      }
      else
      {
         g_DDHGreebleStoreSettings = true;
      }
   }
}
//---------------------------------------------------------------------------------------
// Create greebles
static void ddhgreeble_go( int argc, const char* argv[] )
{

   if( argc < 20 )
   {
      message_dialog( "Insufficient parameters passed to greeble plugin" );
      return;
   }

   // Greeble object
   CGreebler g;

   // Get params
   bool realtime_on = ( strcmp( "0", argv[1] ) != 0 );
   bool preview_only = ( strcmp( "0", argv[2] ) != 0 );

   g.mDoBevel = ( strcmp( "0", argv[3] ) != 0 );

   if( sscanf( argv[4], "%g", &( g.mBevRandomPct )) != 1 )
   {
      message_dialog( "Error parsing random bevel percent (%s)", argv[4] );
      return;
   }
   if( sscanf( argv[5], "%g", &( g.mBevDistancePctMin )) != 1 )
   {
      message_dialog( "Error parsing bevel height minimum percent (%s)", argv[5] );
      return;
   }
   if( sscanf( argv[6], "%g", &( g.mBevDistancePctMax )) != 1 )
   {
      message_dialog( "Error parsing bevel height maximum percent (%s)", argv[6] );
      return;
   }
   if( sscanf( argv[7], "%g", &( g.mBevIndentPctMin )) != 1 )
   {
      message_dialog( "Error parsing bevel minimum indent percent (%s)", argv[7] );
      return;
   }
   if( sscanf( argv[8], "%g", &( g.mBevIndentPctMax )) != 1 )
   {
      message_dialog( "Error parsing bevel maximum indent percent (%s)", argv[8] );
      return;
   }

   g.mDoGreeble = ( strcmp( "0", argv[9] ) != 0 );
   if( sscanf( argv[10], "%g", &( g.mGreRandomPct )) != 1 )
   {
      message_dialog( "Error parsing random greeble percent (%s)", argv[10] );
      return;
   }
   if( sscanf( argv[11], "%d", &( g.mGreNumPerSurfaceMin )) != 1 )
   {
      message_dialog( "Error parsing minimum greebles per surface (%s)", argv[11] );
      return;
   }
   if( sscanf( argv[12], "%d", &( g.mGreNumPerSurfaceMax )) != 1 )
   {
      message_dialog( "Error parsing maximum greebles per surface (%s)", argv[12] );
      return;
   }
   if( sscanf( argv[13], "%g", &( g.mGreHeightPctMin )) != 1 )
   {
      message_dialog( "Error parsing minimum greeble height percent (%s)", argv[13] );
      return;
   }
   if( sscanf( argv[14], "%g", &( g.mGreHeightPctMax )) != 1 )
   {
      message_dialog( "Error parsing maximum greeble height percent (%s)", argv[14] );
      return;
   }
   if( sscanf( argv[15], "%g", &( g.mGreSizePctMin )) != 1 )
   {
      message_dialog( "Error parsing minimum greeble size percent (%s)", argv[15] );
      return;
   }
   if( sscanf( argv[16], "%g", &( g.mGreSizePctMax )) != 1 )
   {
      message_dialog( "Error parsing maximum greeble size percent (%s)", argv[16] );
      return;
   }
   if( sscanf( argv[17], "%g", &( g.mGreRotIncrDegrees )) != 1 )
   {
      message_dialog( "Error parsing greeble rotation increment degrees (%s)", argv[17] );
      return;
   }
   if( sscanf( argv[18], "%g", &( g.mRandSeed )) != 1 )
   {
      message_dialog( "Error parsing random seed (%s)", argv[18] );
      return;
   }

   // The last params are shoved into one spot, delimited by pipes, as we've exceeded the number of params allowed.
   // If the string doesn't have at least 6 chars, it's malformed, so ignore and go with defaults.
   bool save_settings = true;
   g.mUseCustom = false;
   g.mUseStandard = true;
   g.mBevGridPct = 90.0f;
   g.mObjKeepSeparate = false;
   g.mGreCustNoConform = false;
   if( strlen( argv[19] ) > 8 )
   {
      save_settings = ( argv[19][0] != '0' );
      g.mUseCustom = ( argv[19][2] != '0' );
      g.mUseStandard = ( argv[19][4] != '0' );
      g.mObjKeepSeparate = ( argv[19][6] != '0' );
      g.mGreCustNoConform = ( argv[19][8] != '0' );
      char *p = ( char * )( argv[19] ) + 9;
      if( *p == '|' )
      {
         ++p;
      }
      // TODO: If we add more params, parse them out of the pipes, including this one:
      if( sscanf( p, "%g", &( g.mBevGridPct )) != 1 )
      {
         message_dialog( "Error parsing divided bevel frequency (%s)", p );
         return;
      }
   }

   g.mBevRandom = ( g.mBevRandomPct < 100.0f );
   g.mGreRandom = ( g.mGreRandomPct < 100.0f );

   // Adjust percentages
   g.mBevDistancePctMin *= 0.01f;
   g.mBevDistancePctMax *= 0.01f;
   g.mBevIndentPctMin *= 0.01f;
   g.mBevIndentPctMax *= 0.01f;
   g.mBevRandomPct *= 0.01f;
   g.mGreRandomPct *= 0.01f;
   g.mGreHeightPctMin *= 0.01f;
   g.mGreHeightPctMax *= 0.01f;
   g.mGreSizePctMin *= 0.01f;
   g.mGreSizePctMax *= 0.01f;
   g.mBevGridPct *= 0.01f;

   // Obey constraints
   if( g.mBevDistancePctMin - 0.0000005f > g.mBevDistancePctMax )
   {
      g.mBevDistancePctMin = g.mBevDistancePctMax;
      printf( "greeble : minimum bevel height must be less than or equal to maximum bevel height.\n" );
   }
   if( g.mBevIndentPctMin - 0.0000005f > g.mBevIndentPctMax )
   {
      g.mBevIndentPctMin = g.mBevIndentPctMax;
      printf( "greeble : minimum bevel indent must be less than or equal to maximum bevel indent.\n" );
   }
   if( g.mGreHeightPctMin - 0.0000005f > g.mGreHeightPctMax )
   {
      g.mGreHeightPctMin = g.mGreHeightPctMax;
      printf( "greeble : minimum greeble height must be less than or equal to maximum greeble height.\n" );
   }
   if( g.mGreSizePctMin - 0.0000005f > g.mGreSizePctMax )
   {
      g.mGreSizePctMin = g.mGreSizePctMax;
      printf( "greeble : minimum greeble size must be less than or equal to maximum greeble size.\n" );
   }
   if( g.mGreNumPerSurfaceMin > g.mGreNumPerSurfaceMax )
   {
      g.mGreNumPerSurfaceMin = g.mGreNumPerSurfaceMax;
      printf( "greeble : minimum number of greebles per surface must be less than or equal to maximum number of greebles per surface.\n" );
   }

   // Remove the old greeble object
   if( g_DDHGreebleCreatedObjectList != NULL )
   {
      ACObject *obj = ( ACObject * )( g_DDHGreebleCreatedObjectList->data );
      if( object_exists( obj ))
      {
         object_delete( obj );
      }
      g_DDHGreebleCreatedObjectList = NULL;
   }

   List *olist = g.Generate( !preview_only );
   if( olist != NULL )
   {
      // The first object in the list is either the only object, or the group of objects
      ACObject *o = ( ACObject * )( olist->data );
      if( realtime_on )
      {
         g_DDHGreebleCreatedObjectList = olist;
         if( !preview_only )
         {
            ddhgreeble_disable_realtime();
         }
      }
      // Write the fur settings
      if(( !preview_only ) && ( save_settings ))
      {
         char greset[256];
         char gretex[16384];
         strcpy( gretex, "// Greeble settings (1.0)" );
         sprintf( greset, "\n// Beveling on: %s", argv[3] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Bevel frequency: %s", argv[4] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Bevel min height: %s", argv[5] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Bevel max height: %s", argv[6] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Bevel min indent: %s", argv[7] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Bevel max indent: %s", argv[8] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greebling on: %s", argv[9] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble frequency: %s", argv[10] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Min greebles per surface: %s", argv[11] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Max greebles per surface: %s", argv[12] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble min height: %s", argv[13] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble max height: %s", argv[14] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble min size: %s", argv[15] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble max size: %s", argv[16] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Greeble rotation increment: %s", argv[17] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Random seed: %s", argv[18] );
         strcat( gretex, greset );
         sprintf( greset, "\n// Divided bevel: %g", g.mBevGridPct );
         strcat( gretex, greset );
         object_set_userdata( o, gretex );
      }
   }

   find_bounding_box();
   redraw_all();
   display_status();

}
//---------------------------------------------------------------------------------------
// Realtime updating
static void ddhgreeble_sync_realtime( int argc, const char* argv[] )
{

   if( argc < 2 )
   {
      return;
   }

   bool realtime_on = ( strcmp( "0", argv[1] ) != 0 );

   // Realtime updates are off
   if( realtime_on )
   {
      if( nothing_is_selected() )
      {
         tcl_command( ".ddhgreeble.frmopt.ddhgreeble_realtime invoke" );
      }
      else
      {
         ddhgreeble_enable_realtime();
      }
   }
   else
   {
      ddhgreeble_disable_realtime();
   }

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      g_DDHGreebleCreatedObjectList = NULL;

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/greeble.tcl\"", path );
      ac_add_tools_menu_item( "Greeble...", "ddhgreeble_dialog",
                              "Apply greebles to selected surfaces" );
      ac_add_command_full( "ddhgreeble_go", ( void * )ddhgreeble_go,
                            0, "argv", "<arg1> <arg2>",
                           "Create greebles" );
      ac_add_command_full( "ddhgreeble_sync_realtime", ( void * )ddhgreeble_sync_realtime,
                            0, "argv", "<arg1>",
                           "Sync realtime greeble interface" );
      ac_add_command_full( "ddhgreeble_set_save_options", ( void * )ddhgreeble_set_save_options,
                            0, "argv", "<arg1>",
                           "Sets whether to save generated greeble settings to the object's Data field" );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Greeble - version 1.1b - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Greeble - Dennis Hawthorne - Applies greebles to selected surfaces." );
}
//---------------------------------------------------------------------------------------
